# This code aggregates SSURGO soils data. It creates a regional raster from a set of state rasters and projects it to Albers Equal Area Conic.
# It also merges key associated attribute tables selected from the larger list. 
# Author: Emma Tait UVM/FEMC emma.tait@uvm.edu
# Last Updated: 21 September 2020
import arcpy
import os
#arrays to hold files to merge. Info on these tables can be found here https://www.nrcs.usda.gov/wps/portal/nrcs/detail/soils/survey/geo/?cid=nrcs142p2_053631
soil_rasters = []
comp_tbls = []
muaggat_tbls = []
chorizon_tbls = []
moist_tbls = []
month_tbls = []
erode_tbls = []
mapunit_tbls = []
valu1_tbls = []
# define variables
path = r'I:\WorkingProjects\ForestWaterConnection\soils\soils_processing'
boundary = r'I:\WorkingProjects\ForestWaterConnection\HUC_Boundary\HUC12\HUC12.shp'
arcpy.env.workspace = path
arcpy.env.overwriteOutput = True

#get the tables and soils raster from the individual state geodatabases
for folders in arcpy.ListWorkspaces("*", "FileGDB"):
    rstr = folders+'\\MapunitRaster_10m'
    soil_rasters.append(rstr)
    comp_tbls.append(folders+'\\component')
    muaggat_tbls.append(folders+'\\muaggatt')
    chorizon_tbls.append(folders+'\\chorizon')
    moist_tbls.append(folders+'\\cosoilmoist')
    month_tbls.append(folders+'\\comonth')
    erode_tbls.append(folders+'\\coerosionacc')
    mapunit_tbls.append(folders+'\\mapunit')
    valu1_tbls.append(folders+'\\Valu1')

print ('merging rasters and tables')

#merge rasters into single soils raster
region_rstr = arcpy.MosaicToNewRaster_management(soil_rasters, path, 'regional_soil.tif',arcpy.SpatialReference(102039),"32_BIT_SIGNED",'',1,'','FIRST')
print('raster merge complete')

#merge key tables and save as csvs
arcpy.TableToTable_conversion(arcpy.Merge_management(comp_tbls, 'regional_component'),path,'regional_component.csv')
arcpy.TableToTable_conversion(arcpy.Merge_management(muaggat_tbls, 'regional_muaggat'),path,'regional_muaggat.csv')
arcpy.TableToTable_conversion(arcpy.Merge_management(chorizon_tbls, 'regional_chorizon'),path,'regional_chorizon.csv')
arcpy.TableToTable_conversion(arcpy.Merge_management(moist_tbls, 'regional_moist'),path,'regional_moist.csv')
arcpy.TableToTable_conversion(arcpy.Merge_management(month_tbls, 'regional_month'),path,'regional_month.csv')
arcpy.TableToTable_conversion(arcpy.Merge_management(erode_tbls, 'regional_erode'),path,'regional_erode.csv')
arcpy.TableToTable_conversion(arcpy.Merge_management(mapunit_tbls, 'regional_mapunit'),path,'regional_mapunit.csv')
print('table merges complete')

#re-project raster to North America Albers Equal Area Conic and then build an attribute table (the values of the raster are the mapunit IDs)
region_rstr = arcpy.BuildRasterAttributeTable_management(arcpy.ProjectRaster_management('regional_soil.tif','regional_MapunitRaster_10m.tif',arcpy.SpatialReference(102008)))
print('reproject and attribute table building complete')

#change the name of the attribute field to match the original table schema
arcpy.CalculateField_management(arcpy.AddField_management(region_rstr,'mapunit','LONG'),'mapunit','!Value!', 'PYTHON3')
print('script complete')


        
