# This code takes daily rasters of snow depth as derived from SNODAS in snodasProcessing.py and then transforms them into various water yearly summary rasters (meaning Nov/Dec of the year before and Jan-Apr of the year as named (eg a "Year" of 2010 is Nov/Dec 2009 and Jan-Apr 2010):
# Mean depth over all days of water year wy_mean_depth_<YYYY>.tif
# Standard deviation of snow depth over all days of water year inches wy_std_depth_<YYYY>.tif
# Sum of days over water year where depth >8 inches wy_sum_<YYYY>.tif
# Sum of days over water year where there was no snow wy_sum_nosnow_<YYYY>.tif
# Author: Jim Duncan, UVM/FEMC james.duncan@uvm.edu
# Last updated: 4/12/2019

import arcpy
from arcpy.sa import *
from arcpy import env
arcpy.CheckOutExtension("Spatial")
import os
import datetime
#********************for testing********************
import logging
logging.basicConfig(
    format='%(asctime)s %(levelname)-8s %(message)s',
    level=logging.INFO,
    datefmt='%Y-%m-%d %H:%M:%S')
#**********************************************
#List of water years to compute
waterYears=["2005","2006","2007","2008","2009","2010","2011","2012","2013","2014","2015","2016","2017","2018","2019","2020"]
waterYears=["2006"]
#crosswalks month values to naming conventions in the filesystem
monthDict={"11":"11_Nov","12":"12_Dec","1":"01_Jan","2":"02_Feb","3":"03_Mar","4":"04_Apr","5":"05_May","6":"06_Jun","7":"07_Jul","8":"08_Aug","9":"09_Sep","10":"10_Oct"}
outPath="waterYearStats/"
sourceDir="D:/Duncan_SNODAS/testing/"
scratchDir="D:/Duncan_SNODAS/testing/scratch/"
durTemplate="D:/Duncan_SNODAS/durTemplate.tif"

##End of user input

durThisStep=sourceDir+"durThisStep.tif"
durLastStep=sourceDir+"durLastStep.tif"

arcpy.env.workspace=sourceDir
arcpy.env.scratchWorkspace=sourceDir
arcpy.env.overwriteOutput=True
os.chdir(sourceDir)


#Loop through each water year (oct of previous year to April of current year)
for waterYear in waterYears:
    print ("Working on water year "+waterYear)
    cons=[]
    calcList=[] #Hols rasters of days of snow >1 inch for annual summary
    sweCalcList=[] #Holds rasters of SWE for annual summary
    depthCalcList=[] #Holds rasters of daily data that should be used for mean and STD calculation for annual summaries
    
    #reset dur tracking rasters
    print("Resetting dur rasters")
    ##File based approach
    #arcpy.Delete_management(durLastStep)
    #arcpy.Delete_management(durThisStep)
    #arcpy.CopyRaster_management(durTemplate,durLastStep)
    #arcpy.CopyRaster_management(durTemplate,durThisStep)
    #create a duration raster for duration of longest found in this year - this checks to see if this-step has a zero, if it does, check if last-step at this cell was greater than what is already in duration, and update is yes, keep same if no.
    #arcpy.CopyRaster_management(durTemplate, durFinalName)
    #arcpy.CopyRaster_management(durTemplate, durFinalTempName)
    
    ##Object based approach
    #durLastStep=Raster(durTemplate)
    durThisStep=Raster(durTemplate)
    durFinalName=sourceDir+"wy_duration_"+waterYear+".tif"
    durFinalTempName=sourceDir+"wy_duration_"+waterYear+"temp.tif"
    durArray=[Raster(durTemplate)]
    #durFinal=Raster(durTemplate)
    
    print("Looping through months")
    for key, month in monthDict.items():
        #If the month is November or December, we tack this on to the previous water year, otherwise goes to water year matching the year value
        if int(key)>=11:
            newWaterYear=str(int(waterYear)-1)
        elif int(key)<=4:
            newWaterYear=waterYear
        #Only do stuff if we are between November and April
        if (int(key)<=4 or int(key)>=11):
            #Gather list of file names for calculating yearly mean depth and std
            arcpy.env.workspace=sourceDir+newWaterYear+"/"+monthDict[key]
            os.chdir(sourceDir+newWaterYear+"/"+monthDict[key])
            tiffs = arcpy.ListFiles('*.tif')
            for tiff in tiffs:
                if (tiff[:12]=="region_depth"):
                    #add this file to the list of rasters to compute depth from
                    depthCalcList.append(newWaterYear+"/"+monthDict[key]+"/"+tiff)
                    
                    #add this file to the list of conditionals for whether snow was over 1 inch that day
                    #The test is whether the raster was at or above 1 inch depth on this day, 1 if yes, 0 if no. SNODAS is in millimeters depth, so .0254m=1in
                    cons.append("Con('"+newWaterYear+"/"+monthDict[key]+"/"+tiff+"'>25,1,0)")
                    
                    #compare this day's depth to the tracking raster. If 
                if (tiff[:8]=="region_swe"):
                    #add this file to the list of rasters to compute SWE stats from
                    sweCalcList.append(newWaterYear+"/"+monthDict[key]+"/"+tiff)
            #loop through days in a month to build duration raster
            print("Looping through days")
            for i in range(1,32): #range( )does not include the stop number, so have to go to 32
                 #Check that current day is in the dataset
                 if(os.path.isfile("region_depth_"+newWaterYear+key.zfill(2)+str(i).zfill(2)+".tif")==True) :
                    #logging.info("working on "+key+" month and "+str(i)+" day")
                    #arcpy.Delete_management(durLastStep)
                    #arcpy.Delete_management(durFinalTemp)
                    #make previous iteration the last iteration
                    #arcpy.CopyRaster_management(durThisStep,durLastStep)
                    #arcpy.Delete_management(durThisStep)
                    
                    #arcpy.Delete_management(durFinal)
                    #update ThisStep based on current
                    datestamp=newWaterYear+key.zfill(2)+str(i).zfill(2)
                    durLastStep=durThisStep
                    depthRaster=Raster("region_depth_"+datestamp+".tif")
                    durThisStep=Con(depthRaster>0,durLastStep+1,0)
                    #update durFinal based on whether durLastStep or durFinal is larger
                    #durFinalTemp=Con(durFinal<durThisStep,durThisStep,durFinal)
                    #durTemp=Con(durArray[ -1 ]<durThisStep,durThisStep,durArray[-1])
                    #durArray.append(durTemp)
                    durArray.append(depthRaster)
                    
                    #logging.info('con finished')
                    
                    #durFileScratchName=scratchDir+"wy_duration_"+str(i).zfill(2)+key+waterYear+"temp.tif"
                    #durFinalTemp.save(durFileScratchName)
                    
                    #del durFinal
                    #durFinal=durFinalTemp
                    
                    #durFinal.save(durFinalName)
                    #logging.info('day finished\n')
                    #del durFinalTemp
            sys.exit()
    print(waterYear)
    arcpy.env.workspace=sourceDir
    sys.exit()
    durFinal.save(durFinalName)
    #create conditional statements
    cond='+'.join(cons)
      
    #run raster calculator to get year raster of days with snow>1inch (wy_sum_snowdays_yyyy.tif) 
    print ("calculating number of days >1")
    #UNCOMMENT NEXT LINE TO RUN FROM SCRATCH
    arcpy.gp.RasterCalculator_sa(cond, outPath+"wy_sum_snowdays_"+waterYear+".tif")
    # #Add calc for STD over water year, means across years, and std across years. Also go backt
    print ("calculating mean of snow depth over  water year")
    arcpy.env.workspace=sourceDir
    outCellStatistics = CellStatistics(depthCalcList, "MEAN", "DATA")
    outCellStatistics.save(outPath+"wy_mean_depth_"+waterYear+".tif")
    print ("calculating STD of snow depth over  water year")
    arcpy.env.workspace=sourceDir
    outCellStatistics = CellStatistics(depthCalcList, "STD", "DATA")
    outCellStatistics.save(outPath+"wy_std_depth_"+waterYear+".tif")
    print ("calculating max of snow depth over  water year")
    arcpy.env.workspace=sourceDir
    outCellStatistics = CellStatistics(depthCalcList, "MAXIMUM", "DATA")
    outCellStatistics.save(outPath+"wy_max_depth_"+waterYear+".tif")
    print ("calculating mean of SWE over  water year")
    arcpy.env.workspace=sourceDir
    outCellStatistics = CellStatistics(sweCalcList, "MEAN", "DATA",'MULTI_BAND')
    outCellStatistics.save(outPath+"wy_mean_swe"+waterYear+".tif")


# Replace a layer/table view name with a path to a dataset (which can be a layer file) or create the layer/table view within the script
# The following inputs are layers or table views: "snodas_ranPTs_GCMa2_19812010_MonthlyVals"
#arcpy.env.workspace="I:/Administration/jim_docs/McSten/Snowmobile/snodasProcessing/"
#arcpy.gp.Sample_sa(filelist, "I:/Administration/jim_docs/McSten/Snowmobile/snodasStatistics/snodas_ranPTs_GCMa2_19812010_MonthlyVals.shp", "I:/Administration/jim_docs/McSten/Snowmobile/snodasStatistics/MonthlySampleVals", "NEAREST", "FID", "CURRENT_SLICE")
